<?php
/*
 *  This file is part of phynx.

 *  phynx is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.

 *  phynx is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.

 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  2007, 2008, 2009, 2010, Rainer Furtmeier - Rainer@Furtmeier.de
 */
class HTMLForm {
	private $id;
	private $fields;
	private $types;
	private $labels;
	private $table;
	private $options;
	private $values;

	private $saveMode;
	private $saveButtonLabel;
	private $saveButtonBGIcon;
	private $saveButtonSubmit;
	private $saveButtonOnclick;
	private $saveClass;
	private $saveAction;

	private $onSubmit;

	private $spaces;

	private $endLV;

	private $translationClass;

	private $buttons;

	public function __construct($formID, $fields, $title = null){
		$this->id = $formID;
		$this->fields = $fields;
		$this->types = array();
		$this->labels = array();
		$this->options = array();
		$this->values = array();
		$this->endLV = array();
		$this->spaces = array();
		$this->table = new HTMLTable(2, $title);
		$this->saveMode = null;
		$this->onSubmit = null;
		$this->buttons = array();
	}

	public function addSaveDefaultButton($fieldName){
		$B = new Button("als Standard-Wert speichern", "./images/i2/save.gif");
		$B->rme("mUserdata","","setUserdata",array("'DefaultValue$this->id$fieldName'","$('$this->id').$fieldName.value"),"checkResponse(transport);");
		$B->type("icon");
		$B->style("float:right;");
		if(!isset($this->values[$fieldName])) {
			$U = new mUserdata();
			$this->values[$fieldName] = $U->getUDValue("DefaultValue$this->id$fieldName", "");
		}
		$this->buttons[$fieldName] = $B;
	}

	public function getAllFields(){
		return $this->fields;
	}

	public function translate(iTranslation $translationClass){
		$this->translationClass = $translationClass;
		if($translationClass == null) return;

		$labels = $this->translationClass->getLabels();
		#$labelDescriptions = $this->translationClass->getLabelDescriptions();
		#$fieldDescriptions = $this->translationClass->getFieldDescriptions();
		#$this->texts = $this->languageClass->getText();

		$this->table->setCaption($this->translationClass->getEditCaption());
		$this->saveButtonLabel = $this->translationClass->getSaveButtonLabel();

		if($labels != null)
			foreach($labels AS $k => $v)
				$this->setLabel($k, $v);

		/*if($labelDescriptions != null)
			foreach($labelDescriptions AS $k => $v)
				$gui->setLabelDescription($k, $v);

		if($fieldDescriptions != null)
			foreach($fieldDescriptions AS $k => $v)
				$gui->setFieldDescription($k, $v);*/
	}

	public function setSaveMultiCMS($saveButtonLabel, $saveButtonBGIcon, $class, $action){
		$this->saveMode = "multiCMS";
		$this->saveButtonLabel = $saveButtonLabel;
		$this->saveButtonBGIcon = $saveButtonBGIcon;
		$this->saveClass = $class;
		$this->saveAction = $action;
		$this->onSubmit = "multiCMS.formHandler('$this->id'); return false;";
	}

	public function setSaveRMEP($saveButtonLabel, $saveButtonBGIcon, $targetClass, $targetClassId, $targetMethod, $onSuccessFunction){
		$this->saveMode = "rmeP";
		$this->saveButtonLabel = $saveButtonLabel;
		$this->saveButtonBGIcon = $saveButtonBGIcon;
		$this->saveButtonSubmit = "rmeP('$targetClass', '$targetClassId', '$targetMethod', joinFormFieldsToString('$this->id'), '$onSuccessFunction');";
	}

	public function setType($fieldName, $type, $value = null, $options = null){
		$this->types[$fieldName] = $type;
		$this->options[$fieldName] = $options;
		if($value != null) $this->values[$fieldName] = $value;
	}

	public function setValue($fieldName, $value){
		$this->values[$fieldName] = $value;
	}

	public function setLabel($fieldName, $label){
		$this->labels[$fieldName] = $label;
	}

	public function setSaveBericht($berichtClass){
		$this->saveMode = "Bericht";
		$this->saveButtonLabel = "Einstellungen speichern";
		$this->saveButtonBGIcon = "";
		$this->saveClass = "";
		$this->saveAction = "";
		$this->saveButtonOnclick = "saveBericht('".get_class($berichtClass)."');";
	}

	public function insertSpaceAbove($fieldName, $label = ""){
		$this->spaces[$fieldName] = $label;
	}

	/**
	 * @return HTMLTable
	 */
	public function getTable(){
		return $this->table;
	}

	public function addTableEndLV($label, $value){
		$this->endLV[] = array($label, $value);
	}

	public function  __toString() {
		$hiddenFields = "";

		foreach($this->fields as $k => $v){
			if(isset($this->spaces[$v]) AND $this->spaces[$v] != ""){
				$this->table->addRow(array("",$this->spaces[$v]));
				$this->table->addRowStyle("font-weight:bold;");
			}
			if(isset($this->spaces[$v]) AND $this->spaces[$v] == ""){
				$this->table->addRow(array(""));
				$this->table->addRowClass("backgroundColor0");
			}

			$B = "";
			$Input = new HTMLInput(
				$v,
				isset($this->types[$v]) ? $this->types[$v] : "text",
				isset($this->values[$v]) ? $this->values[$v] : null,
				isset($this->options[$v]) ? $this->options[$v] : null);

			if(isset($this->buttons[$v])) {
				$B = $this->buttons[$v];
				if(!isset($this->types[$v]) OR $this->types[$v] == "text" OR $this->types[$v] == "select")
					$Input->style("width:90%;");
			}

			if(!isset($this->types[$v]) OR $this->types[$v] != "hidden")
				$this->table->addLV(
					(isset($this->labels[$v]) ? $this->labels[$v] : ucfirst($v)).":",
					$B.$Input);
			else $hiddenFields .= $Input;
		}
		if($this->saveMode != null)
			switch($this->saveMode){
				case "multiCMS":
					$S = new HTMLInput("submitForm","submit",$this->saveButtonLabel);
					if($this->saveButtonBGIcon != "")
						$S->style("background-image:url($this->saveButtonBGIcon);background-position:98% 50%;background-repeat:no-repeat;");

					$action = new HTMLInput("action", "hidden", $this->saveAction);
					$handler = new HTMLInput("HandlerName", "hidden", $this->saveClass);
					$return = new HTMLInput("returnP", "hidden", "/".$_GET["permalink"]);

					$this->table->addRow(array("",$S.$action.$handler.$return));
				break;

				case "rmeP":
					$S = new HTMLInput("submitForm","button",$this->saveButtonLabel);
					if($this->saveButtonBGIcon != "")
						$S->style("background-image:url($this->saveButtonBGIcon);background-position:98% 50%;background-repeat:no-repeat;");

					$S->onclick($this->saveButtonSubmit);

					$this->table->addRow(array($S));
					$this->table->addRowColspan(1, 2);
				break;

				case "Bericht":
					$S = new HTMLInput("submitForm","button",$this->saveButtonLabel);
					if($this->saveButtonBGIcon != "")
						$S->style("background-image:url($this->saveButtonBGIcon);background-position:98% 50%;background-repeat:no-repeat;");
					$S->onclick($this->saveButtonOnclick);

					$this->table->addRow(array($S));
					$this->table->addRowColspan(1, 2);
				break;
			}

		foreach($this->endLV AS $k => $v)
			$this->table->addLV($v[0],$v[1]);

		$html = "
	<form id=\"$this->id\" ".($this->onSubmit != null ? "onsubmit=\"$this->onSubmit\"" : "").">";

		$html .= $this->table;


		$html .= $hiddenFields."
	</form>";

		return $html;
	}
}
?>
