<?php
/*
 *  This file is part of phynx.

 *  phynx is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.

 *  phynx is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.

 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  2007, 2008, 2009, 2010, Rainer Furtmeier - Rainer@Furtmeier.de
 */
class HTMLInput {
	private $type;
	private $name;
	private $value;
	private $options;
	private $onclick = null;
	private $onchange = null;
	private $style = null;
	private $id = null;
	private $onkeyup = null;
	private $hasFocusEvent = false;
	private $isSelected = false;
	private $isDisabled = false;
	private $isDisplayMode = false;
	private $tabindex;
	private $multiEditOptions;

	public function __construct($name, $type = "text", $value = null, $options = null){
		$this->name = $name;
		$this->type = $type;
		$this->value = $value;
		$this->options = $options;
	}

	public function setType($type){
		$this->type = $type;
	}

	public function activateMultiEdit($targetClass, $targetClassID){
		$this->multiEditOptions = array($targetClass, $targetClassID);
	}

	public function setOptions($options, $labelField = null){
		if(is_object($options) AND $options instanceof Collection AND $labelField != null){
			$this->options = array("0" => "bitte auswählen");

			while($t = $options->getNextEntry())
				$this->options[$t->getID()] = $t->A($labelField);
		} else
			$this->options = $options;
	}

	public function tabindex($index){
		$this->tabindex = $index;
	}

	public function isDisplayMode($b){
		$this->isDisplayMode = $b;
	}

	public function id($id){
		$this->id = $id;
	}

	public function style($style){
		$this->style = $style;
	}

	public function onclick($function){
		$this->onclick = $function;
	}

	public function onchange($function){
		$this->onchange = $function;
	}

	public function onkeyup($function){
		$this->onkeyup = $function;
	}

	public function onEnter($function){
		$this->onkeyup = "if(event.keyCode == 13) ".$function;
	}

	public function hasFocusEvent($bool){
		$this->hasFocusEvent = $bool;
	}

	public function isSelected($bool){
		$this->isSelected = $bool;
	}

	public function setValue($v){
		$this->value = $v;
	}

	public function isDisabled($bool){
		$this->isDisabled = $bool;
	}

	public function  __toString() {
		$style = "";
		if($this->type == "date") $this->style .= "width:87%;";
		if($this->style != null) $style = " style=\"$this->style\"";

		switch($this->type){
			case "multiInput":
				return "<input
					class=\"multiEditInput2\"
					type=\"text\"
					$style
					value=\"".htmlspecialchars($this->value)."\"
					onfocus=\"oldValue = this.value;\"
					id=\"".$this->options[2]."ID".$this->options[1]."\"
					onblur=\"if(oldValue != this.value) saveMultiEditInput('".$this->options[0]."','".$this->options[1]."','".$this->options[2]."');\"
					onkeydown=\"if(event.keyCode == 13) saveMultiEditInput('".$this->options[0]."','".$this->options[1]."','".$this->options[2]."');\"/>";
			break;

			case "textarea":
				if($this->isDisplayMode) return nl2br($this->value);

				return "<textarea
					$style
					name=\"$this->name\"
					".($this->onkeyup != null ? "onkeyup=\"$this->onkeyup\"" : "")."
					".($this->hasFocusEvent ? "onfocus=\"focusMe(this);\" onblur=\"blurMe(this);\"" : "")."
					".($this->id != null ? "id=\"$this->id\"" : "").">$this->value</textarea>";
			break;

			case "date":
			case "text":
			case "hidden":
			case "submit":
			case "button":
			case "password":
			case "checkbox":
				if($this->isDisplayMode) return $this->value;
				
				$cal = "";
				if($this->type == "date") {
					if($this->id == null) $this->id = rand(10000,20000);

					$cal = new Button("","./images/i2/calendar.gif");
					$cal->onclick("Kalender.show(null,null,'$this->id',-365,0,'%d.%m.%y','');");
					$cal->type("icon");
					$cal->className("calendarIcon");
				}

				$value = "value=\"$this->value\"";
				if($this->type == "checkbox") $value = $this->value == "1" ? "checked=\"checked\"" : "";

				return "$cal<input
					$style
					".($this->onclick != null ? "onclick=\"$this->onclick\"" : "")."
					".($this->onkeyup != null ? "onkeyup=\"$this->onkeyup\"" : "")."
					".($this->tabindex != null ? "tabindex=\"$this->tabindex\"" : "")."
					".($this->hasFocusEvent ? "onfocus=\"focusMe(this);\" onblur=\"blurMe(this);\"" : "")."
					name=\"$this->name\"
					".($this->isDisabled ? "disabled=\"disabled\"" : "")."
					type=\"$this->type\"
					".($this->id != null ? "id=\"$this->id\"" : "")."
					$value />";
			break;

			case "option":
				return "<option$style ".($this->isDisabled ? "disabled=\"disabled\"" : "")." ".($this->isSelected ? "selected=\"selected\"" : "")." value=\"$this->value\">$this->name</option>";
			break;

			case "select":
				if($this->isDisplayMode) return is_object($this->options[$this->value]) ? $this->options[$this->value]->__toString() : $this->options[$this->value];

				if($this->multiEditOptions != null){
					$this->onchange("saveMultiEditInput('".$this->multiEditOptions[0]."','".$this->multiEditOptions[1]."','".$this->name."');");
					$this->id($this->name."ID".$this->multiEditOptions[1]);
				}

				$html = "<select$style ".($this->onchange != null ? "onchange=\"$this->onchange\"" : "")." name=\"$this->name\" ".($this->id != null ? "id=\"$this->id\"" : "").">";

				if($this->options != null AND is_array($this->options))
					foreach($this->options AS $k => $v)
						if(!is_object($v)) $html .= "<option ".($this->value == $k ? "selected=\"selected\"" : "")." value=\"$k\">$v</option>";
						else {
							if($this->value == $k)
								$v->isSelected(true);
							$html .= $v;
						}


				$html .= "</select>";

				return $html;
			break;
		}
	}
}
?>
