<?php
/*
 *  This file is part of phynx.

 *  phynx is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.

 *  phynx is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.

 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 *  2007, 2008, 2009, 2010, Rainer Furtmeier - Rainer@Furtmeier.de
 */
class HTMLTable extends UnifiedTable implements iUnifiedTable  {
	#private $numCols;
	#private $caption;
	#private $content = array();
	private $colStyles = array();
	#private $colWidths = array();
	private $rowStyles = array();
	private $rowEvents = array();
	private $rowColspan = array();
	private $cellIDs = array();
	#private $header = array();
	private $cellStyles = array();
	private $cellEvents = array();
	private $hiddenCols = array();
	private $colOrder;
	private $tableStyle;
	private $uID;
	
	private $forms = array();
	
	private $insertSpaceBefore = array();
	private $tabs = array();

	#private $colClass = array();

	function __construct($numCols = 0, $caption = null){
		$this->numCols = $numCols;
		$this->caption = $caption;
	}
	
	function __toString(){
		return $this->getHTML();
	}

	#function setColClass($colNumber,$class){
	#	$this->colClass[$colNumber] = $class;
	#}

	#function setCaption($caption){
	#	$this->caption = $caption;
	#}
	
	function insertSpaceAbove($label = "", $tab = false, $formName = ""){
		if(count($this->insertSpaceBefore) == 0) $this->uID = rand(1000000,9000000);
		
		$this->insertSpaceBefore[count($this->content) - 1] = $label;
		$this->tabs[count($this->content) - 1] = $tab;
		$this->forms[count($this->content) - 1] = $formName;
	}
	
	function setColWidth($colNumber, $width){
		$this->colWidth[$colNumber] = $width.(strpos($width, "px") === false ? "px": "");
	}
	
	function setTableStyle($style){
		$this->tableStyle = $style;
	}
	
	function hideCol($col){
		if(is_array($col)) $this->hiddenCols = $col;
		else $this->hiddenCols[] = $col;
	}
	
	function setColOrder($order){
		if(!is_array($order)) $this->colOrder = array($order);
		else $this->colOrder = $order;
	}

	function addLV($label, $value){
		$this->addRow(array("<label>".$label."</label>",$value));
	}

	function addRowColspan($colNumber, $span){
		if(!isset($this->rowColspan[count($this->content) - 1])) $this->rowColspan[count($this->content) - 1] = array();
		$this->rowColspan[count($this->content) - 1][0] = $colNumber;
		$this->rowColspan[count($this->content) - 1][1] = $span;
	}
	
	function addCellStyle($colNumber, $style){
		if(!isset($this->cellStyles[count($this->content) - 1]))
			$this->cellStyles[count($this->content) - 1] = array();
			
		if(!isset($this->cellStyles[count($this->content) - 1][$colNumber]))
			$this->cellStyles[count($this->content) - 1][$colNumber] = array();

			
		$this->cellStyles[count($this->content) - 1][$colNumber] = $style;
	}
	
	function addCellEvent($colNumber, $event, $action){
		if(!isset($this->cellEvents[count($this->content) - 1]))
			$this->cellEvents[count($this->content) - 1] = array();
			
		if(!isset($this->cellEvents[count($this->content) - 1][$colNumber]))
			$this->cellEvents[count($this->content) - 1][$colNumber] = array();
			
		if(!isset($this->cellEvents[count($this->content) - 1][$colNumber][$event]))
			$this->cellEvents[count($this->content) - 1][$colNumber][$event] = "";

			
		$this->cellEvents[count($this->content) - 1][$colNumber][$event] = $action;
	}
	
	#function addRow($content){
	#	if(!is_array($content)) $content = array($content);
	#	$this->content[] = $content;
	#}
	
	#function addHeaderRow($content){
	#	if(!is_array($content)) $content = array($content);
	#	$this->header = $content;
	#}
	
	function addRowEvent($event, $action){
		if(!isset($this->rowEvents[count($this->content) - 1])) $this->rowEvents[count($this->content) - 1] = array();
		if(!isset($this->rowEvents[count($this->content) - 1][$event])) $this->rowEvents[count($this->content) - 1][$event] = "";
		$this->rowEvents[count($this->content) - 1][$event] .= $action;
	}
	
	function addRowClass($class){
		if(!isset($this->rowClasses[count($this->content) - 1])) $this->rowClasses[count($this->content) - 1] = "";
		$this->rowClasses[count($this->content) - 1] .= " ".$class;
	}
	
	function addColStyle($colNumber, $style){
		if(!isset($this->colStyles[$colNumber])) $this->colStyles[$colNumber] = $style;
		else $this->colStyles[$colNumber] .= $style;
	}
	
	function addRowStyle($style){
		if(!isset($this->rowStyles[count($this->content) - 1])) $this->rowStyles[count($this->content) - 1] = $style;
		else $this->rowStyles[count($this->content) - 1] .= $style;
	}
	
	function addCellID($cellNo, $ID){
		if(!isset($this->cellIDs[count($this->content) - 1])) $this->cellIDs[count($this->content) - 1] = array();
		
		$this->cellIDs[count($this->content) - 1][$cellNo] = $ID;
	}
	
	function getHTML(){
		if($this->content == null) return "";

		$cols = "";
		$rows = "";
		
		for($i = 0; $i < $this->numCols; $i++)
			$cols .= "
				<col ".(isset($this->colWidth[$i+1]) ? "style=\"width:".$this->colWidth[$i+1]."\"" : "")." class=\"".(!isset($this->colClass[$i+1]) ? "backgroundColor".($i%2 + 2)."" : $this->colClass[$i+1])."\" />";
		
		if(count($this->header) > 0){
			$rows .= "
			<tr>";
			
			foreach($this->header as $K => $V){
				if($this->colOrder != null AND isset($this->colOrder[$K]))
					$j = $this->colOrder[$K] - 1;
				else $j = $K;
				
				#$style = "";
				$style = (isset($this->colStyles[$j+1]) ? $this->colStyles[$j+1] : "");
				#if(isset($this->cellStyles[$K][$j+1])) $style .= $this->cellStyles[$K][$j+1];
				
				if($style != "") $style = "style=\"$style\"";
				
				$rows .= "
				<th $style>".(($this->colOrder != null AND isset($this->colOrder[$K])) ? $this->header[$this->colOrder[$K] - 1] : $V)."</th>";
			}
			$rows .= "
			</tr>";
		}
		
		$tab = 1;
		foreach($this->content as $K => $V){

			$events = "";
			if(isset($this->rowEvents[$K]))
				foreach($this->rowEvents[$K] AS $n => $a)
					$events .= "on$n=\"$a\"";
			
			if(isset($this->insertSpaceBefore[$K-1]) AND $this->insertSpaceBefore[$K-1] == "")
			$rows .= "
			<tr>
				<td class=\"backgroundColor0\"></td>
			</tr>";
			
			if(isset($this->insertSpaceBefore[$K-1]) AND $this->insertSpaceBefore[$K-1] != "" AND !$this->tabs[$K-1]) $rows .= "
			<tr>
				<td class=\"backgroundColor0\"></td>
			</tr>
			<tr>
				<td class=\"backgroundColor1\" style=\"font-weight:bold;\" colspan=\"$this->numCols\">".$this->insertSpaceBefore[$K-1]."</td>
			</tr>";
			
			if(isset($this->insertSpaceBefore[$K-1]) AND $this->insertSpaceBefore[$K-1] != "" AND $this->tabs[$K-1]) {
				if($tab > 1) $rows .= "
			</table>
			</form>
			</div>";

				$rows .= "
			<div onclick=\"if($('Tab$this->uID$tab').style.display == 'none') new Effect.BlindDown('Tab$this->uID$tab', {queue: 'end'}); else new Effect.BlindUp('Tab$this->uID$tab', {queue: 'end'});\" class=\"backgroundColor1 Tab borderColor1\">
				<p>".$this->insertSpaceBefore[$K-1]."</p>
			</div>
			<div id=\"Tab$this->uID$tab\" style=\"display:none;\">
			<form ".($this->forms[$K-1] != "" ? "id=\"".$this->forms[$K-1]."\"" : "").">
			<table ".($this->tableStyle != null ? "style=\"$this->tableStyle\"" : "").">
				<colgroup>$cols
				</colgroup>";
				$tab++;	
			}
				
			$rows .= "
			<tr $events ".(isset($this->rowStyles[$K]) ? "style=\"".$this->rowStyles[$K]."\"" : "")." ".(isset($this->rowClasses[$K]) ? "class=\"".$this->rowClasses[$K]."\"" : "").">";
			
			for($l = 0; $l < $this->numCols; $l++){
				
				if($this->colOrder != null AND isset($this->colOrder[$l]))
					$j = $this->colOrder[$l] - 1;
				else $j = $l;
				
				$style = (isset($this->colStyles[$j+1]) ? $this->colStyles[$j+1] : "");
				if(isset($this->cellStyles[$K][$j+1])) $style .= $this->cellStyles[$K][$j+1];
				
				if($style != "") $style = "style=\"$style\"";
				
				$cellEvents = "";
				if(isset($this->cellEvents[$K][$j+1]))
					foreach($this->cellEvents[$K][$j+1] as $on => $ac)
						$cellEvents .= " on$on=\"$ac\"";
				
				$rows .= "
				<td $cellEvents ".((isset($this->cellIDs[$K]) AND isset($this->cellIDs[$K][$j+1])) ? "id=\"".$this->cellIDs[$K][$j+1]."\"" : "")." ".((isset($this->rowColspan[$K]) AND $this->rowColspan[$K][0] == $j+1) ? "colspan=\"".$this->rowColspan[$K][1]."\"" : "")." ".$style.">".(isset($this->content[$K][$j]) ? $this->content[$K][$j] : "")."</td>";
				
				if(isset($this->rowColspan[$K]) AND $this->rowColspan[$K][0] == $j+1)
					$l+= $this->rowColspan[$K][1] - 1;
			}
			$rows .= "
			</tr>";
		}
		
		$R = "
		".($this->caption != null ? "
			<div class=\"backgroundColor1 Tab\">
				<p>$this->caption</p>
			</div>" : "");
		
		if($tab == 1) $R .= "
		<table ".($this->tableStyle != null ? "style=\"$this->tableStyle\"" : "").">
			<colgroup>$cols
			</colgroup>";
		
		$R .= "
			$rows
		</table>";
		if($tab > 1) $R .= "
		</form>
		</div>";
		return $R;
	}
}
?>
