<?php
/*
 *  This file is part of open3A.

 *  open3A is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.

 *  open3A is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.

 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 *  2007, 2008, 2009, 2010, Rainer Furtmeier - Rainer@Furtmeier.de
 */
class Adresse extends PersistentObject implements iDeletable, iXMLExport, iLDAPExport {

	public function getLDAPSchema($client = ""){
		$objectclass = array();
		$objectclass[] = "top";
		$objectclass[] = "person";
		$objectclass[] = "organizationalPerson";
		$objectclass[] = "inetOrgPerson";
		$objectclass[] = "mozillaAbPersonObsolete";
	
		return array(
			"uid" => "{AdresseID}",
			"cn" => "{vorname} {nachname}",
			"sn" => "{nachname}",
			"givenname" => "{vorname}",
			"mail" => "{email}",
			"xmozillausehtmlmail" => "TRUE",
			"telephoneNumber" => "{tel}",
			"mobile" => "{mobil}",
			"title" => "",
			"o" => "{firma}",
			"postOfficeBox" => "{strasse} {nr}",
			"l" => "{ort}",
			"st" => "",
			"postalCode" => "{plz}",
			"countryname" => "{land}",
			"description" => "{bemerkung}",
			"objectclass" => $objectclass);
	}

	// <editor-fold defaultstate="collapsed" desc="__toString">
	function __toString(){
		if($this->A == null) $this->loadMe();
		
		return $this->getHTMLFormattedAddress();
	}
	// </editor-fold>
	
	public function __construct($id){
		$this->myAdapterClass = "AdresseAdapter";
		if($_SESSION["S"]->checkForPlugin("mLDAP"))
			$this->myAdapterClass = "AdresseLDAPAdapter";
	
		parent::__construct($id);
			
		if(!isset($_SESSION["viaInterface"]))
			$this->customize();
	}
	
	public function getA(){
		if($this->A == null) $this->loadMe();
		return $this->A;
	}

	public function newAttributes(){
		$A = parent::newAttributes();
		
		$A->AuftragID = "-1";
		$A->KategorieID = "0";
		$A->type = "default";

		if($this->customizer != null)
			$this->customizer->customizeNewAttributes($this->getClearClass(get_class($this)), $A);
			
		return $A;
	}
	
	public function getHTMLFormattedAddress(){
		if($this->A == null) $this->loadMe();
		if($this->A == null) return "Lieferant unbekannt";
		$r = "";
		if($this->A->firma != "") $r .= $this->A->firma."<br />";
		if($this->A->nachname != "") $r .= $this->A->vorname.($this->A->vorname != "" ? " " : "").$this->A->nachname."<br />";
		$r .= "".$this->A->strasse." ".$this->A->nr."<br />";
		if($this->A->plz != "") $r .= $this->A->plz." ".$this->A->ort.($this->A->land != "" ? "<br />".$this->A->land : "");
		
		return $r;
	}
	
	public function getFormattedAddress($withAnrede = false, $language = "de_DE"){
		if($this->A == null) $this->loadMe();
		$r = "";
		if($this->A->firma != "") $r .= $this->A->firma."\n";
		
		if($this->A->nachname != "") $r .= ($withAnrede ? Util::formatAnrede($language, $this, true)." " : "").$this->A->vorname.($this->A->vorname != "" ? " " : "").$this->A->nachname."\n";
		
		$r .= "".$this->A->strasse." ".$this->A->nr."\n";
		
		if($this->A->plz != "") $r .= $this->A->plz." ".$this->A->ort.($this->A->land != "" ? "\n".$this->A->land : "");
		
		return $r;
	}

	function newMe($checkUserData = true, $output = false){
		$ps = mUserdata::getPluginSpecificData("Adressen");
		
		if(isset($ps["pluginSpecificCanUse1xAdresse"]) AND $this->A->AuftragID == -1) die("<p>Erstellen nicht möglich!<br />Plugin-spezifische Einstellungen aktiv.<br /><br />Sie können das Erstellen von Einträgen im Adressen-Plugin auch verbieten, dann wird die Option \"Adresse neu anlegen\" nicht mehr angezeigt.<br />Neue 1x-Adressen können weiterhin verwendet werden.</p>");
		elseif(!isset($ps["pluginSpecificCanUse1xAdresse"])) mUserdata::checkRestrictionOrDie("cantCreate".str_replace("GUI","",get_class($this)));
		
		return parent::newMe(false, $output);
	}

	function saveMe($checkUserData = true, $output = true){
		$ps = mUserdata::getPluginSpecificData("Adressen");

		if(isset($ps["pluginSpecificCanUse1xAdresse"]) AND $this->A->AuftragID == -1) die("Speichern nicht möglich!");

		parent::saveMe($checkUserData, $output);
	}
	
	public function newFromAdresse($AuftragID){
		$this->loadMe();
		$this->A->AuftragID = $AuftragID;
		$this->A->type = "default";
		return $this->newMe(true, false);
	}
	
	public function getHeader(){
		return ($this->A->firma != "" ? $this->A->firma : $this->A->vorname." ".$this->A->nachname);
	}
}
?>