<?php
/*
 *  This file is part of open3A.

 *  open3A is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.

 *  open3A is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.

 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 *  2007, 2008, 2009, 2010, Rainer Furtmeier - Rainer@Furtmeier.de
 */
class AdressenGUI extends Adressen implements iGUIHTMLMP2, iAutoCompleteHTML, icontextMenu {
	
	protected $gui;
	public $searchFields = array("nachname","vorname","firma","ort","kundennummer");
	public $inAC = false;

	function __construct(){
		parent::__construct();
		$this->customize();
		$bps = $this->getMyBPSData();

		if($bps == -1 OR !isset($bps["AdressBuch"]) OR $bps["AdressBuch"] == 0) $this->addAssocV3("type","=","default");
		else {
			$AB = new AdressBuch($bps["AdressBuch"]);
			if($AB->A("AdressBuchTyp") == "1" OR
				($AB->A("AdressBuchTyp") == "2" AND $AB->A("AdressBuchUserID") == $_SESSION["S"]->getCurrentUser()->getID()))
				$this->addAssocV3("type","=","AB".$bps["AdressBuch"]);
		}

		$this->gui = new HTMLGUI();
		if($bps != -1 AND isset($bps["selectionMode"]) AND strpos($bps["selectionMode"], "Vertrag") !== false){
			$this->gui = new HTMLGUI2();
			$this->gui->setDisplaySide("right");
		}
	}
	
	function getHTML($id, $page){
		try {
			new CRMHTMLGUI();
			$id = -1;
		} catch(ClassNotFoundException $e){ }
		
		$gui = $this->gui;
		
		$gui->VersionCheck("Adressen");

		$gui->setName("Adresse");
		$gui->setObject($this);

		$gui->setParser("firma","AdressenGUI::firmaParser",array("\$sc->vorname","\$sc->nachname","\$aid", "\$type", "\$tel", "\$fax", "\$email", "\$mobil", "\$homepage"));

		$gui->customize($this->customizer);
		$gesamt = $this->loadMultiPageMode($id, $page, 20);

		if(get_class($this) == "AdressenGUI"){
			$_SESSION["BPS"]->setActualClass("adressenMode");
			$_SESSION["BPS"]->unsetACProperty("adressenMode");
		}

		$tab = "";
		if($_SESSION["S"]->checkForPlugin("mAdressBuch") AND $id == -1){
			$bps = $this->getMyBPSData();

			try {
				$AB = new mAdressBuchGUI();
				$AB->addAssocV3("AdressBuchTyp", "=", "1", "AND", "1");
				$AB->addAssocV3("AdressBuchTyp", "=", "2", "OR", "2");
				$AB->addAssocV3("AdressBuchUserID", "=", $_SESSION["S"]->getCurrentUser()->getID(), "AND", "2");

				$options = array();
				$options[0] = "Standard-Adressbuch";

				while($t = $AB->getNextEntry())
					$options[$t->getID()] = $t->A("AdressBuchName");

				$IS = new HTMLInput("selectAbo", "select", isset($bps["AdressBuch"]) ? $bps["AdressBuch"] : "0", $options);
				$IS->onchange("contentManager.loadFrame('contentRight','Adressen','-1','0','_AdressenGUI;AdressBuch:'+this.value);");
				$B = new Button("Adressbücher bearbeiten","./images/i2/edit.png");
				$B->type("icon");
				$B->style("float:right;");
				$B->onclick("contentManager.loadFrame('contentRight','mAdressBuch','-1','0');");

				$tab = new HTMLTable(1);
				$tab->addRow($B.$IS);
			} catch(TableDoesNotExistException $e){
				//do nothing
			}
		}
		
		$gui->isQuickSearchable(str_replace("GUI","",get_class($this)));
		$gui->setMultiPageMode($gesamt, $page, 20, "contentRight", str_replace("GUI","",get_class($this)));
		if($this->collector != null) $gui->setAttributes($this->collector);
		
		$gui->setDisplayGroup("KategorieID");
		$gui->setDisplayGroupParser("AdressenGUI::DGParser");
		
		$gui->setShowAttributes(array("firma"));
		
		$gui->setJSEvent("onNew","function() { reloadRightFrame(); }");
		

		$bps = $this->getMyBPSData();

		$gui->autoCheckSelectionMode(get_class($this));
		return ($id == -1 ? $tab : "").$gui->getBrowserHTML($id);
	}
	
	public static function DGParser($w,$l,$p){
		if($w == 0)
			return "-";
		
		$k = new Kategorie($w);
		return $k->getA() != null ? $k->getA()->name : "Kategorie unbekannt";
	}
	
	public function getACHTML($attributeName, $query){
		$this->inAC = true;
		$gui = $this->gui;
		
		switch($attributeName){
			case "plz":
				$this->addAssocV3("plz","LIKE", $query."%");
				$this->addGroupV3("plz");
				$this->setFieldsV3(array("plz","ort"));
				$this->setLimitV3("10");
				$this->lCV3();
				
				#$gui->setAttributes($this->collector);
				$gui->setObject($this);
				$gui->setShowAttributes(array("plz","ort"));
				echo $gui->getACHTMLBrowser();
			break;
			case "quickSearchAdressen":
				$mode = "quickSearchLoadFrame";
				
				$this->setSearchStringV3($query);
				$this->setSearchFieldsV3($this->searchFields);
				$this->addJoinV3("Kappendix","AdresseID","=","AdresseID");
				$this->setLimitV3("10");
				$this->lCV3();

				$gui->setObject($this);
				#$gui->setAttributes($this->collector);
				$gui->setShowAttributes(array("kundennummer","firma"));
				
				$gui->setParser("firma","AdressenGUI::ACFirmaParser",array("\$sc->nachname","\$sc->vorname"));
				
				$_SESSION["BPS"]->registerClass(get_class($gui));
				$_SESSION["BPS"]->setACProperty("targetFrame","contentLeft");
				$_SESSION["BPS"]->setACProperty("targetPlugin","Adresse");
				$gui->autoCheckSelectionMode(get_class($this));
				$gui->customize($this->customizer);
				echo $gui->getACHTMLBrowser($mode);
			break;

		}
	}
	
	public static function ACFirmaParser($w,$l,$p){
		$s = HTMLGUI::getArrayFromParametersString($p);
		return ($w != "" ? $w.($s[0] != "" ? "<br /><small>$s[1] $s[0]</small>" : "") : "$s[1] $s[0]");
	}
	
	public function getContextMenuHTML($identifier){
		echo "<p style=\"padding:5px;\">Es werden folgende Felder durchsucht:<br /><br />Kundennummer<br />Firma<br />Vorname<br />Nachname<br />Ort</p><p>Sie können Ihre Suchanfrage mit UND verknüpfen.<br />Also z.B. \"Firmenname UND Ort\"</p>";

	}
	
	public function saveContextMenu($identifier, $key){}
	
	public static function firmaParser($w,$a,$p){
		$s = HTMLGUI::getArrayFromParametersString($p);
		
		$symbols = "";
		if($s[8] != "") $symbols .= "<a href=\"$s[8]\"><img class=\"mouseoverFade\" style=\"float:right;margin-left:4px;\" src=\"./images/i2/flowers.gif\" title=\"$s[8]\" /></a>";
		if($s[7] != "") $symbols .= "<img class=\"mouseoverFade\" style=\"float:right;margin-left:4px;\" src=\"./images/i2/mobile.gif\" title=\"$s[7]\" />";
		if($s[6] != "") $symbols .= "<a href=\"mailto:$s[6]\"><img class=\"mouseoverFade\" style=\"float:right;margin-left:4px;\" src=\"./images/i2/email.gif\" title=\"$s[6]\" /></a>";
		if($s[5] != "") $symbols .= "<img class=\"mouseoverFade\" style=\"float:right;margin-left:4px;\" src=\"./images/i2/fax.gif\" title=\"$s[5]\" />";
		if($s[4] != "") $symbols .= "<img class=\"mouseoverFade\" style=\"float:right;margin-left:4px;\" src=\"./images/i2/tel.gif\" title=\"$s[4]\" />";
		
		return $symbols.(($_SESSION["S"]->checkForPlugin("Kunden") AND $s[3] == "default") ? 
		"<img src=\"./images/i2/kunde.png\" title=\"Kundendaten anzeigen/erstellen\" onclick=\"new Ajax.Updater('contentLeft','./interface/loadFrame.php?bps=KundeGUI;AdresseID:$s[2];action:Kappendix&p=Kunde');\" style=\"float:left;margin-right:4px;\" class=\"mouseoverFade\" />" : "")
		.(($_SESSION["S"]->checkForPlugin("Kundenpreise") AND $s[3] == "default") ? "<img src=\"./images/i2/kundenpreis.png\" title=\"Kundenpreise festlegen\" onclick=\"loadFrameV2('contentLeft','Kunde','KundeGUI;AdresseID:$s[2];action:Kundenpreise');/*new Ajax.Updater('contentLeft','./interface/loadFrame.php?bps=KundeGUI;AdresseID:$s[2];action:Kundenpreise&p=Kunde');*/\" style=\"float:left;margin-right:4px;\" class=\"mouseoverFade\" />" : "")
		.(($_SESSION["S"]->checkForPlugin("labelPrinter") AND $s[3] == "default") ? "<img src=\"./images/i2/printer.png\" title=\"Etikette mit Adresse drucken\" onclick=\"rme('labelPrinter','','printEtikette','$s[2]');\" style=\"float:left;margin-right:4px;\" class=\"mouseoverFade\" />" : "")
		.($w != "" ? stripslashes($w).(($s[1] != "" OR $s[0] != "") ? "<br /><small>$s[0] $s[1]</small>" : "") : $s[0]." ".$s[1]);
	}
}
?>