<?php
/*
 *  This file is part of open3A.

 *  open3A is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.

 *  open3A is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.

 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 *  2007, 2008, 2009, 2010, Rainer Furtmeier - Rainer@Furtmeier.de
 */
class mArtikelGUI extends mArtikel implements iGUIHTMLMP2, iAutoCompleteHTML, icontextMenu, iCategoryFilter {

	private $isJoined = false;
	
	function __construct(){
		parent::__construct();
		$bps = $this->getMyBPSData();
		
		$this->setParser("bruttopreis","Util::CLNumberParserZ");
			
		if($_SESSION["S"]->checkForPlugin("Kundenpreise") 
			AND $bps != -1 
			AND $_SESSION["BPS"]->isPropertySet(get_class($this),"kundennummer")){
				
			$this->setParser("kundenPreis","mPosten::numberParser");
				
			$this->addJoinV3("Kundenpreis","ArtikelID","=","ArtikelID");
			$this->addJoinV3("Kundenpreis","kundennummer","=",$bps["kundennummer"]);
			
			$this->isJoined = true;
		}
		
	}
	
	function getAvailableCategories(){
		$kat = new Kategorien();
		$kat->addAssocV3("type","=","2");
		return $kat->getArrayWithKeysAndValues();
	}
	
	function getCategoryFieldName(){
		return "KategorieID";
	}
	
	function getHTML($id, $page){
		
		$gui = new HTMLGUI();
		$gui->VersionCheck("mArtikel");
		
		$this->addOrderV3("KategorieID");
		$this->addOrderV3("name");
		
		if($_SESSION["S"]->checkForPlugin("mFile") OR $_SESSION["S"]->checkForPlugin("mStaffelpreis"))
			$gui->setJSEvent("onNew","function() { reloadRightFrame(); }");
		
		$gui->showFilteredCategoriesWarning($this->filterCategories(),$this->getClearClass());
		
		$gesamt = $this->loadMultiPageMode($id, $page, 0);
		
		$gui->isQuickSearchable(get_parent_class($this));
		
		$gui->setMultiPageMode($gesamt, $page, 0, "contentRight", str_replace("GUI","",get_class($this)));
		if($this->collector != null) $gui->setAttributes($this->collector);
		
		$gui->setName("Artikel");
		$gui->setCollectionOf("Artikel","Artikel");
		
		
		$gui->setDisplayGroup($this->getCategoryFieldName(), $this->getAvailableCategories());
		
		#$gui->setParser("mwst","mArtikelGUI::mwst");
		$gui->setParser("name","mArtikelGUI::nameParser",array("\$sc->bemerkung"));
		$gui->setParser("preis","mArtikelGUI::preisParser",array("\$isBrutto","\$bruttopreis"));
		#$gui->setMode($mode);
		
		$gui->autoCheckSelectionMode(get_class($this));
		
		$shownFields = array("name","preis");
		
		if($this->isJoined) {
			$gui->addColClass("kundenPreis","backgroundColor1");
			$gui->addColStyle("kundenPreis","text-align:right;");
			$gui->setParser("kundenPreis","mArtikelGUI::kPParser");
			
			$shownFields = array_merge(array("kundenPreis"), $shownFields);
		}
		
		$gui->addColStyle("preis","text-align:right;");
		
		$gui->setShowAttributes($shownFields);
		
		return $gui->getBrowserHTML($id);
	}
	
	public static function kPParser($w,$l,$p){
		if($w == "0,00") return "-";
		else return $w;
	}
	
	public static function nameParser($w,$l,$p){
		return stripslashes($w).($p != "" ? "<br /><small>$p</small>" : "");
	}
	
	public static function preisParser($w, $l, $p){
		$s = HTMLGUI::getArrayFromParametersString($p);
		return ($s[0] == "1" ? $s[1]."<br /><small style=\"color:grey;\">Brutto</small>" : $w);
	}
	
	public static function mwst($w){
		return $w."%";
	}

	public function getACHTML($attributeName, $query){
		$gui = new HTMLGUI();
		
		$this->setSearchStringV3($query);
		
		$bps = $this->getMyBPSData();
		$mode = "quickSearchLoadFrame";
		
		if(isset($bps["selectionMode"]) AND $bps["selectionMode"] != "") {
			$mode = "quickSearchSelectionMode";
			#$gui->setMode($bps["mode"]);
			#$this->addAssocV3(($this->isJoined ? "t3" : "t2").".name","LIKE", "%$query%","AND","1");
			$this->setSearchFieldsV3(array(($this->isJoined ? "t3" : "t2").".name","t1.name","bemerkung","beschreibung","artikelnummer"));
		} else $this->setSearchFieldsV3(array("t2.name","t1.name","bemerkung","beschreibung","artikelnummer"));
		#$this->addAssocV3("t2.name","LIKE", "%$query%","AND","1");
		$this->setFieldsV3(array("t1.name","gebinde","bemerkung"));
		$this->addJoinV3("Kategorie","KategorieID","=","KategorieID","ArtikelAttributes");
		
				
		/*$this->addAssocV3("name","LIKE", "%$query%","OR","1");
		$this->addAssocV3("bemerkung","LIKE", "%$query%","OR", "1");
		$this->addAssocV3("beschreibung","LIKE", "%$query%","OR", "1");
		$this->addAssocV3("artikelnummer","LIKE", "%$query%","OR", "1");*/
		
		$this->setLimitV3("10");
		$this->addOrderV3("t1.name","ASC");
		$this->lCV3();
		
		$gui->setAttributes($this->collector);
		$gui->setShowAttributes(array("name","gebinde"));
		$gui->setParser("name","mArtikelGUI::nameParser",array("\$sc->bemerkung"));
		
		$_SESSION["BPS"]->registerClass("HTMLGUI");
		$_SESSION["BPS"]->setACProperty("targetFrame","contentLeft");
		$_SESSION["BPS"]->setACProperty("targetPlugin","Artikel");
		
		$gui->autoCheckSelectionMode(get_class($this));
		echo $gui->getACHTMLBrowser($mode);
	}

	public function getContextMenuHTML($identifier){
		$l = $this->loadLanguageClass("Artikel");
		echo "<p style=\"padding:5px;\">".$l->getSearchHelp()."</p>";
	}
	
	public static function numberParser($w,$l){
		if($l == "load") return number_format($w, 2, ",", "");
		if($l == "store") return str_replace(",",".",$w);
	}
	
	public function saveContextMenu($identifier, $key){}
}
?>
