<?php
/**
 *  This file is part of open3A.

 *  open3A is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.

 *  open3A is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.

 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 *  2007, 2008, 2009, 2010, Rainer Furtmeier - Rainer@Furtmeier.de
 */
class Auftrag extends PersistentObject implements iReNr, iCloneable, iDeletable, iRepeatable, iDesktopLink {
	
	public $GRLBMGUI = null;
	public $AdresseObj = null;

	// <editor-fold defaultstate="collapsed" desc="checkUserRestrictions">
	function checkUserRestrictions($pSpecData = null){
		if($pSpecData == null) $pSpecData = mUserdata::getPluginSpecificData("Auftraege");

		if(isset($pSpecData["pluginSpecificCanOnlyEditOwn"]))
			if($this->A("UserID") != $_SESSION["S"]->getCurrentUser()->getID()) return;

		$hasRestrictions = false;
		foreach($pSpecData as $key => $value)
			if(strstr($key,"pluginSpecificCanSeeAuftraegeFrom"))
				$hasRestrictions = true;

		if($hasRestrictions AND !isset($pSpecData["pluginSpecificCanSeeAuftraegeFrom".$this->A("UserID")]))
			die();
	}
	// </editor-fold>
	
	// <editor-fold defaultstate="collapsed" desc="getLabel">
	function getLabel(){
		return "Standard: JahrNummer, z.B. 08001";
	}
	// </editor-fold>

	// <editor-fold defaultstate="collapsed" desc="newWithDefaultValues">
	function newWithDefaultValues($AdresseID = null){
		$this->A = $this->newAttributes();
		$this->A->UserID = $_SESSION["S"]->getCurrentUser()->getID();
		$this->A->auftragDatum = time();
		$id = $this->newMe(true, false);
		$this->forceReload();

		if($AdresseID != null)
			$this->getAdresseCopy($AdresseID);

		return $id;
	}
	// </editor-fold>

	/*function getA(){
		return $this->A;
	}*/

	// <editor-fold defaultstate="collapsed" desc="sendGRLBMToCustomer">
	public function sendGRLBMToCustomer($GRLBMID, $method){
		$_SESSION["BPS"]->setProperty("Brief","GRLBMID", $GRLBMID);

		switch($method){
			case "email":
				$this->sendViaEmail("", "", false);
			break;

			case "sign":

			break;

			case "mail":

			break;

			case "print":

			break;

			case "none":
			break;
		}
	}
	// </editor-fold>

	// <editor-fold defaultstate="collapsed" desc="getGRLBMToCustomerOptions">
	public static function getGRLBMToCustomerOptions($value = null){
		$a = array("none" => "kein Rechnungsversand", "email" => "E-Mail", "sign" => "E-Mail signiert via PixelLetter", "mail" => "Brief via PixelLetter", "print" => "Drucken");

		if($value != null) return $a[$value];
		return $a;
	}
	// </editor-fold>

	// <editor-fold defaultstate="collapsed" desc="getRepeatableActions">
	public function getRepeatableActions(){
		return array("cloneForRepeatable" => "kopieren");
	}
	// </editor-fold>

	// <editor-fold defaultstate="collapsed" desc="cloneForRepeatable">
	public function cloneForRepeatable(Repeatable $R){
		$this->cloneMe(date("d.m.Y",$R->getNextDate()));
	}
	// </editor-fold>

	// <editor-fold defaultstate="collapsed" desc="cloneMe">
	public function cloneMe($useDate = 0){
		$this->loadMe();
		$oldID = $this->ID;
		$this->A->auftragDatum = time();
		$newAuftragID = $this->newMe();
		
		$_SESSION["BPS"]->registerClass("cloneGRLBM");
		$_SESSION["BPS"]->setACProperty("AuftragID",$newAuftragID);
		if($useDate != 0) {
			$_SESSION["BPS"]->setACProperty("datum",$useDate);
			$_SESSION["BPS"]->setACProperty("lieferDatum",$useDate);
		}
		
		$_SESSION["BPS"]->setActualClass("mGRLBMGUI");
		$_SESSION["BPS"]->setACProperty("type",-1);
		$mG = new mGRLBMGUI();
		ob_start();
		$mG->cloneAllToAuftrag($oldID);
		ob_end_clean();
		echo $newAuftragID;
	}
	// </editor-fold>

	// <editor-fold defaultstate="collapsed" desc="deleteMe">
	function deleteMe(){
		$mP = new anyC();
		$mP->setCollectionOf("GRLBM");
		$mP->addAssocV3("AuftragID","=",$this->ID);
		$mP->lCV3();
		
		if($mP->numLoaded() > 0)
			die("alert:AuftraegeMessages.A002");
		
		parent::deleteMe();
	}
	// </editor-fold>

	// <editor-fold defaultstate="collapsed" desc="updateAdressID">
	function updateAdressID($newID){
		if($this->ID == 0) return;
		if($this->A == null) $this->loadMe();
		
		$this->A->AdresseID = $newID;
		$this->A->kundennummer = -2;
		$this->A->UStIdNr = "";
		$this->saveMe(true, false);
	}
	// </editor-fold>

	// <editor-fold defaultstate="collapsed" desc="getLetter">
	function getLetter($type, $copy = false){
		if($this->A ==  null AND $this->ID != -1) $this->loadMe();
		$brief = new Brief();
		$brief->setStammdaten(Stammdaten::getActiveStammdaten());
		
		$Gid = null;
		$_SESSION["BPS"]->setActualClass("GRLBMGUI");
		$bps = $_SESSION["BPS"]->getAllProperties("GRLBMGUI");
		if($bps != -1 AND isset($bps["myID"]))
			$Gid = $bps["myID"];
		
			
		$_SESSION["BPS"]->setActualClass("Brief");
		$bps = $_SESSION["BPS"]->getAllProperties();
		if($bps != -1 AND isset($bps["GRLBMID"])) {
			$Gid = $bps["GRLBMID"];
			#$_SESSION["BPS"]->unsetACProperty("GRLBMID");
		}
		
		if($Gid == null) die("Fehler, keine GRLBMID!!");
		
		$G = new GRLBMGUI($Gid);
		$G->isCopy = $copy;
		$brief->isCopy = $copy;
		$brief->setGRLBM($G);

		$ad = new AdresseGUI(
		(($G->getA()->isL == "1" OR $G->getA()->isA == "1")
		AND $G->getA()->lieferAdresseID != "0") ? 
			$G->getA()->lieferAdresseID : 
			$this->A->AdresseID);

		$brief->setAdresse($ad);
		$brief->setTextbausteinOben(Textbaustein::getFakeTextbaustein($G->getA()->textbausteinOben));
		
		$posten = new mPosten();
		$posten->addAssocV3("GRLBMID","=",$G->getID());
		$posten->lCV3();
		$posten->setLetterType($G->getMyPrefix());
		$brief->setPosten($posten);
				
		$brief->setZahlungsbedingungen(Textbaustein::getFakeTextbaustein($G->getA()->zahlungsbedingungen));
		$brief->setTextbausteinUnten(Textbaustein::getFakeTextbaustein($G->getA()->textbausteinUnten));
		
		//für Dateiname
		$brief->nummer = $G->getA()->nummer;
		$brief->datum = $G->getA()->datum;
		$brief->kunde = ($ad->getA()->firma != "" ? $ad->getA()->firma : $ad->getA()->vorname." ".$ad->getA()->nachname);
		$brief->type = $G->getMyPrefix();
		
		$brief->rabatt = $G->getA()->rabatt;
		$brief->leasingrate = $G->getA()->leasingrate;
		$brief->rabattInW = $G->getA()->rabattInW;

		return $brief;
	}
	// </editor-fold>

	// <editor-fold defaultstate="collapsed" desc="getAdresseCopy">
	protected function getAdresseCopy($AdresseID){
		$this->loadMe();
		$_SESSION["messages"]->addMessage("creating Adresse for Auftrag ".$this->ID." of Adresse $AdresseID");
		$p = new Adresse($AdresseID);
		$newAdresseID = $p->newFromAdresse($this->ID);	
			
		$mKApp = new mKappendix();
		$mKApp->setAssocV3("AdresseID","=",$AdresseID);
		$KApp = $mKApp->getNextEntry();
		if($KApp != null) {
			$this->forceReload();
			$this->A->kundennummer = $KApp->getA()->kundennummer;
			$this->A->UStIdNr = $KApp->getA()->UStIdNr;
			$this->saveMe(true, false);
		}
	}
	// </editor-fold>

	// <editor-fold defaultstate="collapsed" desc="getNextNumber">
	public static function getNextNumber($type){
		$startNumber = $re_nr = "".date("y")."001";
		
		$_SESSION["BPS"]->setActualClass("mGRLBMGUI");
		$_SESSION["BPS"]->setACProperty("type",$type);
		$n = new mGRLBMGUI();
		
		$re_nr = $n->getIncrementedField("nummer");
		if($re_nr == 1) $re_nr = $startNumber;

		if(date("y") < 10) $re_nr = "0".$re_nr;

		$re_nr2 = $re_nr."";
		if(date("y") > $re_nr2{0}.$re_nr2{1}) $re_nr = $startNumber;
		
		return $re_nr;
	}
	// </editor-fold>

	// <editor-fold defaultstate="collapsed" desc="createGRLBM">
	public function createGRLBM($type, $returnID = false){
		
		$this->loadMe();
		
		$reNrClass = mStammdaten::getReNrTemplate();
		
		$c = new $reNrClass(-1);
		
		$re_nr = $c->getNextNumber($type);
		
		$G = new GRLBM(-1);
		$GA = $G->newAttributes();
		
		$GA->nummer = $re_nr;
		$f = "is".$type;
		
		if($type != "G" AND $type != "R" AND $type != "A" AND $type != "L")
			$GA->isWhat = $type;
		else $GA->$f = "1";
			
		$GA->AuftragID = $this->ID;
		
		$GA->datum = Util::CLDateParser(time());#date("d.m.Y");
		$GA->lieferDatum = Util::CLDateParser(time());#date("d.m.Y");
		
		$GA->prefix = mStammdaten::getActiveStammdaten()->getPrefix($type);
		
		if($this->A->kundennummer != "-2"){
			$KApp = new mKappendix();
			$KApp->setAssocV3("kundennummer","=",$this->A->kundennummer);
			$KA = $KApp->getNextEntry();
			
			if($KA != null AND $type != "Kalk"){
				$f1 = $type."TextbausteinOben";
				$f2 = $type."TextbausteinUnten";
				$f3 = $type."Zahlungsbedingungen";

				if($type == "R" AND $KA->A("KappendixEinzugsermaechtigung") === "1" AND $KA->A("KappendixEinzugsermaechtigungAltZBTB") != null)
					$f3 = "KappendixEinzugsermaechtigungAltZBTB";

				$GA->textbausteinOben = Textbaustein::getTextOf($KA->getA()->$f1);
				$GA->textbausteinUnten = Textbaustein::getTextOf($KA->getA()->$f2);
				$GA->zahlungsbedingungen = Textbaustein::getTextOf($KA->getA()->$f3);
				
				$GA->textbausteinObenID = $KA->getA()->$f1;
				$GA->textbausteinUntenID = $KA->getA()->$f2;
				$GA->zahlungsbedingungenID = $KA->getA()->$f3;
				
				if(isset($GA->zahlungsziel) AND isset($KA->getA()->KappendixZahlungsziel))
					$GA->zahlungsziel = (time() + $KA->getA()->KappendixZahlungsziel * 3600 * 24);#date("d.m.Y");
				
			} else if($KA != null AND $type == "Kalk"){
				$GA->textbausteinOben = "";
				$GA->textbausteinUnten = "";
				$GA->zahlungsbedingungen = "";
			}
		} else {
			if($type != "Kalk"){
				$_SESSION["messages"]->addMessage("getting standard Textbausteine... ");
				
				$mTB = new Textbausteine();
				list($id,$l) = $mTB->getTBs("oben",$type, true);
				$GA->textbausteinOben = Textbaustein::getTextOf($id[0]);
				$GA->textbausteinObenID = $id[0];
				
				$mTB2 = new Textbausteine();
				list($id2,$l) = $mTB2->getTBs("unten",$type, true);
				$GA->textbausteinUnten = Textbaustein::getTextOf($id2[0]);
				$GA->textbausteinUntenID = $id2[0];
				
				$mTB3 = new Textbausteine();
				list($id3,$l) = $mTB3->getTBs("zahlungsbedingungen",$type, true);
				$GA->zahlungsbedingungen = Textbaustein::getTextOf($id3[0]);
				$GA->zahlungsbedingungenID = $id3[0];
				
				if(isset($GA->zahlungsziel))
					$GA->zahlungsziel = (time() + 14 * 3600 * 24);#date("d.m.Y");
				
			}	
		}
		
		
		$G->setA($GA);
		$newID = $G->newMe();
		
		$this->A->auftragDatum = time();
		$this->saveMe(true, false);
		
		if($returnID) return $newID;
		return $re_nr.":".Stammdaten::getLongType($type);
	}
	// </editor-fold>

	// <editor-fold defaultstate="collapsed" desc="sendViaEmail">
	function sendViaEmail($Subject = "", $Body = "", $die = true){

		$G = new GRLBM($_SESSION["BPS"]->getProperty("Brief","GRLBMID"));
		$brief = $this->getLetter("", false);

		$filename = $brief->generate(true);
		$Stammdaten = mStammdaten::getActiveStammdaten();
		$AnAdresse = new Adresse($this->A->AdresseID);

		$recipient = AuftragGUI::getEMailRecipient($G, $AnAdresse, true, $die);
		list($OSubject, $OBody) = AuftragGUI::getEMailTBs($AnAdresse, $Stammdaten, $G, $die);

		if($Subject == "") $Subject = $OSubject;
		if($Body == "") $Body = $OBody;

		list($fromName, $from) = AuftragGUI::getEMailSender($Stammdaten, $die);


	    $mail = new htmlMimeMail5();
	    $mail->setFrom($fromName." <".$from.">");
	    if(!ini_get('safe_mode')) $mail->setReturnPath($from);
	    $mail->setSubject(utf8_decode($Subject));

		$ud = new mUserdata();
		if($_SESSION["S"]->getCurrentUser()->getA()->UserEmail != "" AND $ud->getUDValue("BCCToUser", "false") == "true")
			$mail->setBcc($_SESSION["S"]->getCurrentUser()->getA()->UserEmail);

		$mail->addAttachment(
	    	new fileAttachment(
	    		$filename,
	    		'application/pdf',
	    		new Base64Encoding())
	    );


		$ud = new mUserdata();
    	if($_SESSION["S"]->checkForPlugin("mFile") AND $ud->getUDValue("sendBelegViaEmailAttachments", "false") == "true"){
			$D = new mDateiGUI();
			$D->addAssocV3("DateiClassID","=",$_SESSION["BPS"]->getProperty("GRLBMGUI","myID"));
			$D->addAssocV3("DateiClass","=","GRLBM");

			while($f = $D->getNextEntry())
				$mail->addAttachment(new fileAttachment($f->getA()->DateiPath));

    	}

	    if(strpos($Body, "<html") !== false AND strpos($Body, "</html>") !== false) $mail->setHTML(utf8_decode($Body));
		else $mail->setText(utf8_decode($Body));

	    $adressen = array($recipient);
	    #if($altEMailAddress == "")
			#$adressen[] = $AnAdresse->A("email");
	    #else $adressen[] = $altEMailAddress;
	    if(!$mail->send($adressen)){
	    	if($die) die("error:AuftraegeMessages.E003");
			else throw new Exception("E-Mail: Failed to send e-mail!");
		}
		unlink($filename);

		if($die) echo "message:AuftraegeMessages.M001";

		return true;
	}
	// </editor-fold>

	// <editor-fold defaultstate="collapsed" desc="updateDatum">
	public function updateDatum($newDatum){
		$this->loadMe();

		if($newDatum > $this->A->auftragDatum){
			$this->A->auftragDatum = $newDatum;
			$this->saveMe();
			$_SESSION["messages"]->addMessage("updating date of Auftrag... $newDatum, ".$this->A->auftragDatum);
		} $_SESSION["messages"]->addMessage("updating date of Auftrag... no update necessary");
	}
	// </editor-fold>

	// <editor-fold defaultstate="collapsed" desc="saveMultiEditField">
	protected function saveMultiEditField($field,$value){
		if($this->A == null) $this->loadMe();
		$this->A->$field = $value;
		$this->saveMe();
	}
	// </editor-fold>
	/*
	public function copysugarCRMContactForAuftrag($contactID){
		$C = new sugarCRMContact($contactID);
		$C->loadMe();
		
		$A = new Adresse(-1);
		
		$AA = $A->newAttributes();
		$AA->AuftragID = $this->ID;
		foreach(PMReflector::getAttributesArrayAnyObject($AA) as $K => $V)
			if($C->A($V) != null AND isset($AA->$V)) $AA->$V = $C->A($V);
		
		$A->setA($AA);
		$A->newMe(true, true);
	}*/

	// <editor-fold defaultstate="collapsed" desc="getDesktopLink">
	public function getDesktopLink(){
		$Adresse = new Adresse($this->A("AdresseID"));

		return array("edit",$Adresse->A("firma") == "" ? $Adresse->A("vorname")." ".$Adresse->A("nachname") : $Adresse->A("firma"));
	}
	// </editor-fold>
}
?>
