<?php
/*
 *  This file is part of open3A.

 *  open3A is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.

 *  open3A is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.

 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 *  2007, 2008, 2009, 2010, Rainer Furtmeier - Rainer@Furtmeier.de
 */
class GRLBM extends PersistentObject implements iCloneable, iRepeatable, iDeletable2 {

	public $AdresseObj = null;
	public $isCopy = false;
	protected $letterType = "";
	public $CustomizerTeilzahlungen = false;
	
	public function getOnDeleteEvent(){
		return "function() { reloadRightFrame(); reloadLeftFrame(); }";
	}
	
	public function getOnDeleteQuestion(){
		return "Soll dieser Beleg und alle Positionen wirklich gelöscht werden?";
	}
	
	function __construct($ID){
		$this->myAdapterClass = "GRLBMAdapter";
		parent::__construct($ID);
		$this->setParser("datum","Util::CLDateParser");
		$this->setParser("lieferDatum","Util::CLDateParser");
		$this->setParser("rabatt","mPosten::numberParser");
		$this->setParser("rabattInW","mPosten::numberParser");
		$this->setParser("leasingrate","mPosten::numberParser");
		$this->setParser("payedWithSkonto","mPosten::numberParser");
		$this->setParser("GRLBMpayedDate","Util::CLDateParserE");
		
		$this->customize();
	}
	
	public function getRepeatableActions(){
		$t = array("cloneForRepeatable" => "kopieren", "cloneAndMail" => "kopieren und per E-Mail an Kunde verschicken");
		#if($_SESSION["S"]->checkForPlugin("mDrucker")) {
		#	$t["cloneAndPrintOriginal"] = "kopieren und Original drucken";
		#	$t["cloneAndPrintWithCopy"] = "kopieren und Original und Kopie drucken";
		#}
		
		return $t;
	}
	
	public function getSumOfPosten($returnArray = false, $saveValues = false){
		$this->loadMe();
		$aC = new anyC();
		$aC->setCollectionOf("Posten");
		$aC->setAssocV3("GRLBMID","=",$this->ID);
		
		$gesamt_netto_array = array();
		
		$ges_netto = 0;
		$ges_brutto = 0;
		$ges_mwst = 0;
		$mwst = array();
		while($t = $aC->getNextEntry()){
			$rabatt = 1;
			if(isset($t->getA()->rabatt)) $rabatt = (100 - $t->getA()->rabatt) / 100;

			if(!isset($gesamt_netto_array[$t->getA()->mwst])) $gesamt_netto_array[$t->getA()->mwst] = 0;
			$gesamt_netto_array[$t->getA()->mwst] += $t->getA()->menge * $t->getA()->preis * $rabatt;

			#$ges_mwst += $t->getA()->preis * ($t->getA()->mwst / 100);
			$ges_brutto += ($t->getA()->isBrutto == "1" ? $t->getA()->bruttopreis * $t->getA()->menge * $rabatt : $t->getA()->preis * $t->getA()->menge * (100 + $t->getA()->mwst) / 100 * $rabatt) ;
			
			if(array_search($t->getA()->mwst, $mwst) === false) $mwst[] = $t->getA()->mwst;
		}

		if($aC->numLoaded() == 0 AND !$returnArray) return null;

		foreach($gesamt_netto_array AS $key => $value)
			$ges_mwst += Util::kRound($value * ($key / 100),2);
		
		$ges_brutto = Util::kRound($ges_brutto);
		$ges_mwst = Util::kRound($ges_mwst);
		

		if($this->A->isAbschlussrechnung == "1"){
			
			$oC = new anyC();
			$oC->setCollectionOf("GRLBM");
			$oC->addAssocV3("AuftragID","=",$this->A->AuftragID);
			$oC->addAssocV3("GRLBMID", "!=" , $this->ID);
			
			while($r = $oC->getNextEntry()){
				$v = $r->getSumOfPosten(true);
				$ges_brutto -= $v[2];
				$ges_mwst -= $v[1];
			}
		}
		
		$ges_netto = $ges_brutto - $ges_mwst;

		if($saveValues) {
			$this->changeA("nettobetrag", $ges_netto);
			$this->changeA("bruttobetrag", $ges_brutto);
			$this->changeA("steuern", $ges_mwst);
			$this->saveMe();
		}

		if(!$returnArray) return $ges_brutto;
		else return array($ges_netto, $ges_mwst, $ges_brutto, $mwst);
	}
	
	public function cloneForRepeatable(Repeatable $R){
		$this->loadMe();

		$_SESSION["BPS"]->registerClass("cloneGRLBM");
		$_SESSION["BPS"]->setACProperty("datum", date("d.m.Y", $R->getNextDate()));
		$_SESSION["BPS"]->setACProperty("lieferDatum", date("d.m.Y", $R->getNextDate()));
		if($R->A("RepeatableInterval") == "1Month")
			$_SESSION["BPS"]->setACProperty("GRLBMCustomField3", date("m/Y", $R->getNextDate()));
		if($R->A("RepeatableInterval") == "1Year")
			$_SESSION["BPS"]->setACProperty("GRLBMCustomField3", date("Y", $R->getNextDate()));

		return $this->cloneMe(true);
	}
	
	private function printMe($copy){
		if($this->A == null) $this->loadMe();
		
		$Auftrag = new AuftragGUI($this->A->AuftragID);
		$Auftrag->loadMe();
		$_SESSION["BPS"]->registerClass("GRLBMGUI");
		$_SESSION["BPS"]->setACProperty("myID", $this->ID);
		
		$_SESSION["BPS"]->registerClass("mGRLBMGUI");
		$_SESSION["BPS"]->setACProperty("type", $this->getMyPrefix());
		
		$Auftrag->printLetter($copy ? "true" : "false");
	}

	public function cloneAndPrintOriginal(Repeatable $R){
		$newID = $this->cloneForRepeatable($R);

		$G = new GRLBM($newID);
		$G->printMe(false);
	}

	public function cloneAndMail(Repeatable $R){
		$newID = $this->cloneForRepeatable($R);

		$_SESSION["BPS"]->setProperty("Brief","GRLBMID",$newID);

		$G = new GRLBM($newID);

		$A = new AuftragGUI($G->A("AuftragID"));
		try {
			$A->sendViaEmail("","",false);
		} catch(Exception $e){
			return "ERROR: ".$e->getMessage()."\n";
		}
	}
	
	public function cloneAndPrintWithCopy(Repeatable $R){
		$newID = $this->cloneForRepeatable($R);
		
		$G = new GRLBM($newID);
		$G->printMe(false);
		$G->printMe(true);
	}
	
	function cloneMe($quiet = false){
		if($this->A == null) $this->loadMe();
		
		$_SESSION["BPS"]->setActualClass("clone".get_class($this));
		$bps = $_SESSION["BPS"]->getAllProperties();
		if(count($bps) > 0 AND $bps != -1)
			foreach($bps AS $key => $value)
				if(isset($this->A->$key))
					$this->A->$key = $value;
		
		
		$this->A->nummer = mStammdaten::getNextNumberFor($this->getMyPrefix());
		if(!isset($bps["datum"])) $this->A->datum = Util::CLDateParser(time());#date("d.m.Y");
		if(!isset($bps["lieferDatum"])) $this->A->lieferDatum = Util::CLDateParser(time());#date("d.m.Y");
		$this->A->isPrinted = 0;
		$this->A->isPayed = 0;
		$oldGRLBMID = $this->ID;
		$newGRLBMID = $this->newMe();
		
		$_SESSION["BPS"]->registerClass("clonePosten");
		$_SESSION["BPS"]->setACProperty("GRLBMID",$newGRLBMID);
		
		$mP = new mPosten();
		$mP->cloneAllToGRLBM($oldGRLBMID);
		if(!$quiet) echo $this->A->AuftragID;
		
		$Auftrag = new Auftrag($this->A->AuftragID);
		$Auftrag->updateDatum($this->A->datum);
		
		return $newGRLBMID;
	}
	
	function deleteMe(){
		$mP = new anyC();
		$mP->setCollectionOf("Posten");
		$mP->addAssocV3("GRLBMID","=",$this->ID);
		#$mP->lCV3();
		
		/*if($mP->numLoaded() > 0)
			die("alert:AuftraegeMessages.A001");*/
		
		while($t = $mP->getNextEntry())
			$t->deleteMe();
		
		parent::deleteMe();
	}
	
	function getA(){
		if($this->A == null) $this->loadMe();
		return $this->A;
	}
	
	function updateTBs($o, $u, $zb){
		$this->A->textbausteinOben = $o;
		$this->A->textbausteinUnten = $u;
		$this->A->zahlungsbedingungen = $zb;
		$this->saveMe();
	}
	
	protected function getProvisionCopy($ProvisionID){
		$p = new Posten(-1);
		$p->newFromProvision($ProvisionID,$this->ID);
	}
	
	public function getPostenCopy($ArtikelID){
		$this->loadMe();
		if($this->A->isPayed == "1")
			die("alert:AuftraegeMessages.A004");
			
		$_SESSION["messages"]->addMessage("creating Posten for Auftrag ".$this->ID." of Artikel $ArtikelID");
		$p = new Posten(-1);
		return $p->newFromArtikel($ArtikelID,$this->ID);
	}

	protected function copyPostenFrom($fromId){
		$ps = new anyC();
		$ps->setCollectionOf("Posten");
		$ps->addAssocV3("GRLBMID","=",$fromId);
		$ps->addOrderV3("PostenID");
		while(($t = $ps->getNextEntry())){
			$A = $t->getA();
			$A->GRLBMID = $this->ID;
			
			$nP = new Posten(-1, false);
			$nP->recalcNetto = false;
			$nP->setA($A);
			$nP->newMe();
		}
	}

	public function deleteAllPosten(){
		$ac = new anyC();
		$ac->setCollectionOf("Posten");
		$ac->addAssocV3("GRLBMID", "=", $this->ID);

		while($t = $ac->getNextEntry())
			$t->deleteMe();
	}

	public function addPosten($artikelName, $einheit, $anzahl, $preis, $mwst, $beschreibung = "", $isBrutto = false){
		$P = new Posten(-1, false);
		$A = $P->newAttributes();
		
		$A->GRLBMID = $this->ID;
		$A->name = $artikelName;
		$A->gebinde = $einheit;
		$A->preis = $preis;
		$A->mwst = $mwst;
		$A->menge = $anzahl;
		$A->beschreibung = $beschreibung;

		if($isBrutto == false AND isset($A->isBrutto)) $A->isBrutto = "0";

		if($isBrutto AND isset($A->isBrutto)){
			$A->isBrutto = "1";
			$A->bruttopreis = $preis;
			$A->preis = $preis / ((100 + $mwst) / 100);
			$P->recalcNetto = false;
		}
		

		$P->setA($A);
		return $P->newMe(false);
	}

	public function setTextbaustein($type, $text, $save = true){
		if($type != "textbausteinUnten" AND $type != "textbausteinOben" AND $type != "zahlungsbedingungen") return;
		$tID = $type."ID";

		$this->loadMe();

		$this->A->$tID = 0;
		$this->A->$type = $text;
		
		if($save) $this->saveMe();
	}

	public function setLieferdatum($newDatum, $save = true){
		$this->loadMe();

		$this->A->lieferDatum = $newDatum;

		if($save) $this->saveMe();
	}
	
	protected function copyPostenByTypeAndNumber($fromNumber, $fromType){
		$ac = new anyC();
		$ac->setCollectionOf("GRLBM");
		$ac->addAssocV3("is$fromType","=","1");
		$ac->addAssocV3("nummer","=",$fromNumber);
		
		$id = $ac->getNextEntry();
		
		if($id == null) die("error:AuftraegeMessages.E017");
		if($ac->numLoaded() > 1) die("error:AuftraegeMessages.E018");
		$this->copyPostenFrom($id->getID());
	}
	
	protected function markMeAsUnprinted(){
		if($this->A == null) $this->loadMe();
		$this->A->isPrinted = 0;
		$this->saveMe();
	}
	
	public function markMeAsPrinted(){
		if($this->A == null) $this->loadMe();
		$this->A->isPrinted = 1;
		$this->saveMe();
	}
	
	public function setPayed($p, $skonto = "0,00", $datum = "0", $isTeilzahlung = "false", $TeilzahlungBetrag = "", $save = true){

		$pSpecData = mUserdata::getPluginSpecificData("Auftraege");
		
		if(!isset($pSpecData["pluginSpecificCanSetPayed"])) return "error:AuftraegeMessages.E001";

		$this->loadMe();
		if($isTeilzahlung == "false"){
			$this->A->isPayed = $p == "true" ? 1 : 0;
			$this->A->GRLBMpayedDate = $datum;#Util::CLDateParser($datum,"store");
			$this->A->payedWithSkonto = Util::parseFloat("de_DE", $skonto);
		} else {
			$betrag = !is_numeric($TeilzahlungBetrag) ? Util::CLNumberParser($TeilzahlungBetrag, "store") : $TeilzahlungBetrag;
			$this->A->GRLBMTeilzahlungen .= $betrag.";".Util::CLDateParser($datum,"store")."\n";
			$this->A->GRLBMTeilzahlungenSumme += $betrag;
		}
		if($save) $this->saveMe();
	}
	
	public function deleteTeilzahlung($zeile){
		$pSpecData = mUserdata::getPluginSpecificData("Auftraege");
		
		if(!isset($pSpecData["pluginSpecificCanSetPayed"])) die("error:AuftraegeMessages.E001");
		
		$t = trim($this->A("GRLBMTeilzahlungen"));
		$t2 = "";
		$s = explode("\n",$t);
		for($i = 0; $i < count($s); $i++){
			if($zeile != $i){
				$t2 .= $s[$i]."\n";
			} else {
				$b = explode(";",$s[$i]);
				$this->A->GRLBMTeilzahlungenSumme -= $b[0] * 1;
			}
		}
		$this->A->GRLBMTeilzahlungen = $t2;
		$this->saveMe(true, true);
	}
	
	public function saveMe($checkuserdata = true, $output = false){
		
		$oldMe = new GRLBM($this->ID);
		$oldMe->loadMe();
		
		if($this->A->isPrinted != $oldMe->getA()->isPrinted) $print = true;
		else $print = false;
		
		if($this->A->isPayed == "1" AND $this->A->isR == "1" AND $oldMe->getA()->isPayed == "1" AND !$print)
			die("alert:AuftraegeMessages.A006");
	
		if($this->A->textbausteinObenID != "0"){
			$T = new Textbaustein($this->A->textbausteinObenID);
			$this->A->textbausteinOben = $T->A("text");
		}

		if($this->A->textbausteinUntenID != "0"){
			$T = new Textbaustein($this->A->textbausteinUntenID);
			$this->A->textbausteinUnten = $T->A("text");
		}
		
		if($this->A->zahlungsbedingungenID != "0"){
			$T = new Textbaustein($this->A->zahlungsbedingungenID);
			$this->A->zahlungsbedingungen = $T->A("text");
		}

		parent::saveMe($checkuserdata, $output);
	}
	
	public function getMyPrefix(){
		if($this->A == null) $this->loadMe();

		if($this->A->isR == "1") return "R";
		if($this->A->isL == "1") return "L";
		if($this->A->printAB == "1") return "B";
		if($this->A->isA == "1") return "A";
		if($this->A->isG == "1") return "G";
		if($this->A->isM == "1") return "M";
		if($this->A->isWhat == "Kalk") return "Kalk";
		
		return "-1";
	}
}
?>
