<?php
/*
 *  This file is part of open3A.

 *  open3A is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.

 *  open3A is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.

 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 *  2007, 2008, 2009, 2010, Rainer Furtmeier - Rainer@Furtmeier.de
 */
class Posten extends PersistentObject {
	public $parsers;
	public $recalcNetto = true;
	
	public function getA(){
		return $this->A;
	}
	
	function __construct($id, $parsers = true){
		parent::__construct($id);
		//für Interface
		$this->parsers = $parsers;
		if($parsers){
			$this->setParser("menge","Util::CLNumberParserZ");
			$this->setParser("preis","Util::CLNumberParserZ");
			$this->setParser("EK1","Util::CLNumberParserZ");
			$this->setParser("EK2","Util::CLNumberParserZ");
		}
		/*$this->setParser("menge","mPosten::numberParser");
		$this->setParser("preis","mPosten::numberParser");
		$this->setParser("EK1","mPosten::numberParser");
		$this->setParser("EK2","mPosten::numberParser");
		$this->setParser("mwst","mPosten::numberParser");*/
		$this->customize();
	}
	
	public function newAttributes(){
		$A = parent::newAttributes();

		if($this->customizer != null)
			$this->customizer->customizeNewAttributes($this->getClearClass(get_class($this)), $A);
			
		if($_SESSION["S"]->checkForPlugin("mBrutto"))
			$A->isBrutto = "1";

		return $A;
	}
	
	public function newFromProvision($ProvisionID,$GRLBMID){
		$this->A = $this->newAttributes();
		
		$prov = new Provision($ProvisionID);
		$prov->loadMe();
		$provA = $prov->getA();
		
		if($provA->GRLBMID != -1) {
			$R = new GRLBM($provA->GRLBMID);
			$R->loadMe();
		}
		$provA->provisionBetrag = str_replace(",",".",$provA->provisionBetrag);
		
		$this->A->GRLBMID = $GRLBMID;
		if($provA->GRLBMID != -1) $this->A->name = "Provision zu Rechnung ".$R->getA()->nummer;
		else $this->A->name = "Provision";
		$this->A->menge = 1;
		$this->A->bemerkung = "";
		$this->A->oldArtikelID = $ProvisionID;
		$skonto = 0;
		if($provA->GRLBMID != -1) $skonto = $provA->provisionBetrag * $R->getA()->payedWithSkonto / 100;
		$this->A->preis = number_format(round($provA->provisionBetrag - $skonto + 0.0000000000001,2),2,",","");
		$this->newMe();

		$provA->RechnungID = $GRLBMID;
		$prov->saveMe();
	}
	
	public function newFromArtikel($ArtikelID, $GRLBMID, $menge = 1){
		$this->A = $this->newAttributes();
		$a = new Artikel($ArtikelID);
		
		$this->recalcNetto = false;
		
		#$_SESSION["messages"]->addMessage("loading Artikel $ArtikelID");
		$aAttr = $a->getA();
		$aArray = PMReflector::getAttributesArray($aAttr);
		$pArray = PMReflector::getAttributesArray($this->A);

		$inters = array_intersect($aArray, $pArray);
		
		#$_SESSION["messages"]->addMessage("attributes of Artikel and Posten: ".implode(", ",$inters)."");

		foreach($inters AS $key => $value)
			$this->A->$value = $aAttr->$value;
	
		
		$_SESSION["BPS"]->setActualClass("mArtikelGUI");
		$bps = $_SESSION["BPS"]->getAllProperties();
		
		if($bps != -1 AND $_SESSION["BPS"]->isACPropertySet("kundennummer")){
			$Ks = new Kundenpreise();
			$Ks->addAssocV3("t1.kundennummer","=",$bps["kundennummer"]);
			$Ks->addAssocV3("t1.ArtikelID","=",$ArtikelID);
			$K = $Ks->getNextEntry();
			
			if($K != null) {
				$this->A->preis = $K->getA()->kundenPreis;
				$this->recalcNetto = true;
			}
		}
		
		$this->A->GRLBMID = $GRLBMID;
		$this->A->bemerkung = "";
		$this->A->menge = $menge;
		$this->A->oldArtikelID = $ArtikelID;
		#for($i=0;$i<count($pArray);$i++)
		#	$_SESSION["messages"]->addMessage("\$this->A->".$pArray[$i]." = ".$this->A->$pArray[$i]);
		
		
		#$_SESSION["messages"]->addMessage("saving new Posten");
		
		if($_SESSION["S"]->checkForPlugin("mMultiLanguage") OR isset($_SESSION["viaInterface"])){
			$GRLBM = new GRLBM($GRLBMID);
			$Auftrag = new Auftrag($GRLBM->A("AuftragID"));
			$Adresse = new Adresse($Auftrag->A("AdresseID"));
					
			if($Adresse->A("AdresseSpracheID") != "0"){
				$ML = new mMultiLanguageGUI();
				$ML->addAssocV3("MultiLanguageClass","=","Artikel");
				$ML->addAssocV3("MultiLanguageClassID","=","$ArtikelID");
				
				$ML->addAssocV3("MultiLanguageSpracheID","=",$Adresse->A("AdresseSpracheID"));
				
				while($T = $ML->getNextEntry())
					$this->changeA($T->A("MultiLanguageClassField"),$T->A("MultiLanguageValue"));
			}
		}
		
		return $this->newMe();
	}
	
	protected function saveMultiEditField($field,$value){
		if($field != "preis") $this->recalcNetto = false;
		
		if($this->A == null) $this->loadMe();
		$this->A->$field = $value;
		$this->saveMe();
	}

	private function createArtikel(){
		$a = new Artikel(-1);

		$aAttr = $a->newAttributes();
		$aArray = PMReflector::getAttributesArray($aAttr);
		$pArray = PMReflector::getAttributesArray($this->A);

		$inters = array_intersect($aArray, $pArray);

		foreach($inters AS $key => $value)
			$aAttr->$value = $this->A->$value;

		$a->setA($aAttr);
		return $a->newMe();
	}

	public function newMe($checkuserdata = true, $output = false){
		if($this->A->createArtikel == "1")
			$this->A->oldArtikelID = $this->createArtikel();

		if($this->A->isBrutto == "1" AND $this->recalcNetto){
			$this->A->bruttopreis = Util::CLNumberParserZ($this->A->preis,"store");
			$this->setParser("preis","Util::nothingParser");
			$this->A->preis = $this->A->bruttopreis / (1 + $this->A->mwst / 100);
		}

		$G = new GRLBM($this->A->GRLBMID);
		$G->loadMe();
		
		if($G->getA()->isPayed == "1" AND $G->getA()->isR == "1")
			die("alert:AuftraegeMessages.A004");

		$newid = parent::newMe($checkuserdata, $output);

		$G = new GRLBM($this->A("GRLBMID"));
		$G->getSumOfPosten(false, true);

		return $newid;
	}
	
	public function saveMe($checkuserdata = true, $output = false){
		if($this->A->createArtikel == "1")
			$this->A->oldArtikelID = $this->createArtikel();

		if($this->A->isBrutto == "1" AND $this->recalcNetto){
			$this->A->bruttopreis = Util::CLNumberParserZ($this->A->preis,"store");
			$this->setParser("preis","Util::nothingParser");
			$this->A->preis = $this->A->bruttopreis / (1 + $this->A->mwst / 100);
		}
		
		$G = new GRLBM($this->A->GRLBMID);
		$G->loadMe();
		
		if($G->getA()->isPayed == "1" AND $G->getA()->isR == "1")
			die("alert:AuftraegeMessages.A005");

		parent::saveMe($checkuserdata, $output);

		$G = new GRLBM($this->A("GRLBMID"));
		$G->getSumOfPosten(false, true);
	}
	
	public function deleteMe(){
		if($this->A == null) $this->loadMe();
		
		$G = new GRLBM($this->A->GRLBMID);
		$G->loadMe();
		
		if($G->getA()->isPayed == "1" AND $G->getA()->isR == "1")
			die("alert:AuftraegeMessages.A003");
		
		parent::deleteMe();
	}
	
	public function cloneMe(){
		$this->setParser("menge","Util::nothingParser");
		$this->setParser("preis","Util::nothingParser");
		#$this->setParser("bruttopreis","Util::nothingParser");
		$this->setParser("EK1","Util::nothingParser");
		$this->setParser("EK2","Util::nothingParser");
		
		#$this->forceReload();
		
		$_SESSION["BPS"]->setActualClass("clone".get_class($this));
		$bps = $_SESSION["BPS"]->getAllProperties();
		foreach($bps AS $key => $value)
			if(isset($this->A->$key))
				$this->A->$key = $value;
				
		$this->recalcNetto = false;
		return $this->newMe();
	}
}
?>
