<?php
/*
 *  This file is part of open3A.

 *  open3A is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.

 *  open3A is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.

 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 *  2007, 2008, 2009, 2010, Rainer Furtmeier - Rainer@Furtmeier.de
 */
class mPosten extends anyC implements iFPDF {
	private $letterType = "";
	
	function __construct() {
		$this->setCollectionOf("Posten");
		$this->addOrderV3("PostenID");
	}
	
	public function setLetterType($type){
		$this->letterType = $type;
	}
	
	function getCollector(){
		return $this->collector;
	}
	
	public static function numberParser($w,$l){
		if($l == "load") return number_format($w, 2, ",", "");
		if($l == "store") return str_replace(",",".",$w);
	}

	function cloneAllToGRLBM($GRLBMID){
		$this->addAssocV3("GRLBMID","=",$GRLBMID);

		while(($t = $this->getNextEntry()))
			$t->cloneMe();
	}
	
	function addVirtualPosten($menge, $einheit, $bezeichnung, $beschreibung, $preis, $mwst, $isBrutto, $bruttopreis){
		$this->lCV3();
		
		$P = new Posten(-1);
		$PA = $P->newAttributes();
		$PA->name = $bezeichnung;
		$PA->gebinde = $einheit;
		$PA->preis = $preis;
		$PA->menge = $menge;
		$PA->mwst = $mwst;
		$PA->beschreibung = $beschreibung;
		$PA->isBrutto = $isBrutto;
		$PA->bruttopreis = $bruttopreis;
		
		$P->setA($PA);
		$this->collector[] = $P;
	}
	
	public function getFPDF(FPDF $fpdf){
		if($this->letterType == "M") return;
		$userHiddenFields = mUserdata::getHides("Artikel");
		
		$fpdf->gesamtEK1 = 0;
		$fpdf->gesamtEK2 = 0;
		$fpdf->gesamt_netto = array();
		$fpdf->gesamt_brutto = 0;
		
		$fpdf->printPDFHeader();

		$PC = $this->getCollector();
		
		#$j = 1;
		for($i=0;$i<count($PC);$i++){

			$fpdf->SetFont($fpdf->fontPositionen[0], $fpdf->fontPositionen[1], $fpdf->fontPositionen[2]);
			
			$fpdf->currentArticle = $PC[$i];
			
			$A = $PC[$i]->getA();
			
			$mwstFaktor = 1 + $A->mwst / 100;

			$einzelpreis = $A->preis * 1 * ($fpdf->showBruttoPreise ? $mwstFaktor : 1);
			
			if($fpdf->showBruttoPreise AND $A->isBrutto == "1")
				$einzelpreis = $A->bruttopreis * 1;
			elseif(!$fpdf->showBruttoPreise AND $A->isBrutto == "1") 
				$einzelpreis = $A->bruttopreis / (1 + $A->mwst / 100);

			$dispEinzelpreis = $einzelpreis;
			
			$rabatt = 1;
			if(isset($A->rabatt)) $rabatt = (100 - $A->rabatt) / 100;
			if(isset($A->rabatt) AND $A->rabatt != 0) $einzelpreis *= $rabatt;

			#if($A->isBrutto == "1") $A->preis = $A->preis / (1 + $A->mwst / 100);
			if($A->isBrutto == "1")
				$A->preis = $A->bruttopreis / (1 + $A->mwst / 100);
			
			#die($A->preis);
			if($i != 0)
				$fpdf->setXY($fpdf->GetX(),$fpdf->GetY()+2);
				
			if($this->letterType != "Kalk" AND $fpdf->widthPosition)
				$fpdf->Cell($fpdf->widthPosition, 5, $i + 1, 0, 0, "R");
			
			if($fpdf->widthMenge)
				$fpdf->Cell($fpdf->widthMenge, 5, Util::formatNumber($fpdf->language, $A->menge * 1, 2, true, false), 0, 0, "R");
			
			if($this->letterType != "Kalk" AND $fpdf->widthEinheit)
				$fpdf->Cell($fpdf->widthEinheit, 5, utf8_decode($A->gebinde), 0, 0, "L");
				
			if($this->letterType != "Kalk" AND $fpdf->widthArtikelnummer)
				$fpdf->Cell($fpdf->widthArtikelnummer, 5, utf8_decode($A->artikelnummer), 0, 0, "L");
				
			$xbN = $fpdf->getX();
			
			if($fpdf->widthBezeichnung)
				$fpdf->MultiCell($fpdf->widthBezeichnung, 5, utf8_decode(Util::conv_euro($A->name)), 0, "L");
				
			$yaN = $fpdf->getY();
			
			if($A->beschreibung != "" AND $fpdf->widthBezeichnung) {
				$fpdf->SetFont($fpdf->fontPositionenBeschreibung[0], $fpdf->fontPositionenBeschreibung[1], $fpdf->fontPositionenBeschreibung[2]);
				#$fpdf->Cell(19,5,"",0,0,"R");
				#if($this->letterType != "Kalk") $fpdf->Cell(28,5,"",0,0,"L");
				$fpdf->SetXY($xbN, $yaN);
				$fpdf->MultiCell($fpdf->widthBezeichnung,3,utf8_decode(Util::conv_euro($A->beschreibung)),0,"L",0);
				#$j++;
				$fpdf->SetFont($fpdf->fontPositionen[0], $fpdf->fontPositionen[1], $fpdf->fontPositionen[2]);
				$yaN = $fpdf->getY();
			}
			
			$fpdf->setXY($xbN + $fpdf->widthBezeichnung, $yaN - 5);
			if($this->letterType == "R" OR $this->letterType == "A" OR $this->letterType == "G"){
				if($fpdf->widthEinzelpreis)
					$fpdf->Cell($fpdf->widthEinzelpreis, 5, (!(!$fpdf->showNullPreise AND $dispEinzelpreis == 0)) ? Util::conv_euro(Util::formatCurrency($fpdf->language, $dispEinzelpreis, true, $fpdf->showDezimalstellen)) : "", 0, 0, "R");

				if($fpdf->widthRabatt AND isset($A->rabatt))
					$fpdf->Cell($fpdf->widthRabatt, 5, $A->rabatt * 1 != 0 ? Util::conv_euro(Util::formatNumber($fpdf->language, $A->rabatt * 1, 2))."%" : "", 0, 0, "R");
				
				if($fpdf->widthRabattpreis AND isset($A->rabatt))
					$fpdf->Cell($fpdf->widthRabattpreis, 5, Util::conv_euro(Util::formatCurrency($fpdf->language, $einzelpreis, true)), 0, 0, "R");
				
				if($fpdf->widthGesamt)
					$fpdf->Cell($fpdf->widthGesamt, 5, (!(!$fpdf->showNullPreise AND $einzelpreis == 0)) ? Util::conv_euro(Util::formatCurrency($fpdf->language,$A->menge * $einzelpreis, true)) : "", 0, 0, "R");
			}
			if($this->letterType == "L"){
				#$fpdf->Cell(35,5,"",0,0,"R");
				$fpdf->Cell(0,5,"",0,0,"R");
			}
			if($this->letterType == "Kalk"){
				if($fpdf->widthEK1)
					$fpdf->Cell($fpdf->widthEK1, 5, (!isset($userHiddenFields["EK1"]) ? Util::conv_euro(Util::formatCurrency($fpdf->language,$A->EK1 * 1,true)) : ""), 0, 0, "R");
					
				if($fpdf->widthEK2)
					$fpdf->Cell($fpdf->widthEK2, 5, (!isset($userHiddenFields["EK2"]) ? Util::conv_euro(Util::formatCurrency($fpdf->language,$A->EK2 * 1,true)) : ""), 0, 0, "R");
				
				if($fpdf->widthVK)
					$fpdf->Cell($fpdf->widthVK,5,Util::conv_euro(Util::formatCurrency($fpdf->language,$A->preis  * 1,true)),0,0,"R");
				
				$fpdf->gesamtEK1 += $A->EK1 * $A->menge;
				$fpdf->gesamtEK2 += $A->EK2 * $A->menge;
			}
			$fpdf->Ln();
			#$j++;
			if(!isset($fpdf->gesamt_netto[$A->mwst])) $fpdf->gesamt_netto[$A->mwst] = 0;
			$fpdf->gesamt_netto[$A->mwst] += $A->menge * $A->preis * $rabatt;
			
			if($A->isBrutto == "0") $fpdf->gesamt_brutto += $A->preis * (1 + $A->mwst / 100) * $A->menge * $rabatt;
			else $fpdf->gesamt_brutto += $A->bruttopreis * $A->menge * $rabatt;

			$fpdf->currentArticle = null;
			if($fpdf->GetY() > $fpdf->h - $fpdf->marginBottom - 10) {
				/*$fpdf->Line($fpdf->marginLeft , $fpdf->getY(), 210-$fpdf->marginRight , $fpdf->getY());
				
				$fpdf->Cell(0,5,utf8_decode($fpdf->labelFortsetzung)." ".($fpdf->PageNo()+1),0,0,"R");
				*/
				$fpdf->AddPage();
				#$fpdf->Cell(0,5,utf8_decode($fpdf->labelUebertrag).": ".Util::conv_euro(Util::formatCurrency($fpdf->language, array_sum($fpdf->gesamt_netto) * 1,true)),0,0,"R");
				#$fpdf->Ln();
				#$fpdf->printPDFHeader();
				#$j = 0;
			}
		}
		$fpdf->ln(1);
		
		if(isset($PC[0]) AND $this->letterType == "R"){
			$G = new GRLBM($PC[0]->getA()->GRLBMID);

			if($G->getA()->versandkosten != 0) {
				$parsedMwSt = $G->getA()->versandkostenMwSt;
				$mwstFaktor = 1 + $parsedMwSt / 100;
				if(!isset($fpdf->gesamt_netto[$parsedMwSt])) $fpdf->gesamt_netto[$parsedMwSt] = 0;
				$fpdf->gesamt_netto[$parsedMwSt] += $G->getA()->versandkosten;

				$fpdf->gesamt_brutto += $G->getA()->versandkosten * (1 + $G->getA()->versandkostenMwSt / 100);

				$fpdf->SetDrawColor(100, 100, 100);
				$fpdf->Line($fpdf->marginLeft , $fpdf->getY(), 210-$fpdf->marginRight , $fpdf->getY());
				$fpdf->SetDrawColor(0, 0, 0);
				$fpdf->Cell($xbN - $fpdf->getLeftMargin(), 5, "", 0, 0, "L");
				$fpdf->Cell($fpdf->widthBezeichnung, 5, utf8_decode($fpdf->labelVersandkosten), 0, 0, "L");
				$fpdf->Cell($fpdf->widthEinzelpreis, 5, Util::conv_euro(Util::formatCurrency($fpdf->language, $G->getA()->versandkosten * 1 * ($fpdf->showBruttoPreise ? $mwstFaktor : 1), true)), 0, 0, "R");
				$fpdf->Cell($fpdf->widthGesamt, 5, Util::conv_euro(Util::formatCurrency($fpdf->language, $G->getA()->versandkosten * 1 * ($fpdf->showBruttoPreise ? $mwstFaktor : 1), true)), 0, 0, "R");
				$fpdf->ln();
			}
		}

		$fpdf->Line($fpdf->marginLeft , $fpdf->getY(), 210-$fpdf->marginRight , $fpdf->getY());
		$fpdf->printGesamt($this->letterType);

	}
}
?>